 # Name :          	CompLayerScene ###
# Date :	      	  06 16 2012
# Description :   	C2LAYSCENE
#                   Place the Component on a new layer named from the component
#                 	Layer name includes prefix A##- where ## is 2 digit no (01...99)
#                 	A new Drawing Page named A-## will be created.
#                 	A copy of the Board will be added and moved to 200,0,200 for the Drawing Page
#				  	        The copy will be rotated so long dim is on red axis
#                 	The view will be changed to zoom in on the Board @ 200,0,200
#				  	        If user has MakeOrthoViews.rb installed then ortho views are created
#                 	The drawing page will be updated
#				  	        All instances of the component will be placed on the new layer
#                   
#                   Added NC2SCENE to make layer and scene for Nested Component
#                   Added GP2SCENE to make Sub Assembly scene for grouped components
#                   Added C2SCNE to make scene for Component already placed on its Named Layer  
#                   Added SETPREF to allow user to set Preferences
#
# Author :       	  Keith Krueger #kbkrueger2 at g mail dot com# (with a big help from plugin forum:)
# 

require('sketchup.rb')

module K²WS::Comp2LayerScene

  # Set Constantants
  C2LAYSCENE = "Comp2Layer/Scene"
  NC2SCENE = "NestedComp2Scene"
  GP2SCENE = "SubAssy2Scene"
  C2SCNE = "Comp2Scene"
  SETPREF = "Set Comp2Scene Pref"
  
  class << self
    
    ############ Use this to place NEW COMPONENT on its OWN LAYER with Option to CREATE SCENE
    
    def comp2_layer_scene_command 
      # Model Information
      self.get_model_info
      entity = @ss.first
      ent_def = entity.definition
      self.read_data
      
      # This Section Provides the Layer No Input as Text
      # Turn layer names (Axx- where xx = any digit (0-9)) into a list array
      if @ans[0] == "Yes"
        num_layers = @model.layers.find_all { |layer| layer.name =~ /\AA\d\d\-/} 
        num_layer_names = num_layers.map { |layer| layer.name }
        num_layer_names.sort!
        length_n = num_layer_names.length
        #Debug UI.messagebox("num_layer_names = "  + num_layer_names.to_s)
        if length_n >0
          lname = num_layer_names[length_n-1]
          #Debug  UI.messagebox("lname = "  + lname)
          name_no = lname[1,2].to_i
          next_name_no = (name_no + 1).to_s
          if next_name_no.length < 2 
              layerno = "0" + next_name_no
          else
              layerno = next_name_no
          end # if next_name_no
        else
          layerno = "01" #Provides Starting No for New Dwg
        end # if length
      end # if @ans[0]
      
      # This section gets a Component Name from User
      cName=@ss[0].definition.name
      ps = "Enter Component Name "
      dv = cName
      ts = "Get Name for Component"
      @user_name = get_user_name(ps,dv,ts)
      if @user_name != cName then @ss[0].definition.name = @user_name end
      
      # Test name to determine if unique
      while @ss[0].definition.name != @user_name
        UI.messagebox("Layer \"#{@user_name}\" exists,make unique.")
        dv = @user_name
        @user_name = self.get_user_name(ps,dv,ts)
        @ss[0].definition.name = @user_name
      end # while @ss[0]
      
      # Create new layer for Part Assembly
      if @ans[0] == "Yes"
        @layer_name = "A" + layerno + "-" + @user_name
      else
        @layer_name = @user_name
      end # if @ans[0] == "Yes"
      #Debug UI.messagebox"Layer Name  = " + @layer_name
      
      # Test name to determine if unique
      while @layers[@layer_name]
        UI.messagebox("Layer \"#{@layer_name}\" exists,make unique.")
        ps = "Make Layer Name Unique!!"
        dv = @layer_name
        ts = "Nested Component Layer Name"
        @layer_name = self.get_user_name(ps,dv,ts)
      end # while @layers
      
      Sketchup.active_model.start_operation "Comp2_Layer_Scene"
      
      self.create_layer
      
      #Add Component instances to new layer	
      @model.selection.add(entity.definition.instances)
      nosel=@ss.length
      #layers = model.layers
      i = 0          
      loop do
        #Debug  UI.messagebox("length = " + nosel.to_s)
        entity=@ss[i-1]
        entity.layer=(@layers[@layer_name])
        i=i + 1
        break if i==nosel          
      end # loop

      # Add page for Main View if not already exists
      mv_name = @ans[3].strip
      pagename=[]
      pgAssy = "no"
      @pages.each {|p|
        pagename = p.name
        if pagename == mv_name
          pgAssy = "yes"
        end #if 
      } # end each loop
      
      if pgAssy == "no"
        @pages.add mv_name
      end # if  
             
      # Make New Part Component visible on Main View Page   
      page = @pages[mv_name]
      page.set_visibility(@layers[@layer_name],true)
      
      #Ask user if Drawing Scene should be added
      result=UI.messagebox "Add Scene for Comp Drawing? ", MB_YESNO 			
      if result==6  #Add Scene for Drawing and place copy of component in Scene
        #Add page for drawing
        ps = "Enter Page Name for Component Scene"
        if @ans[0] ==  "Yes" and @ans[1] == "Yes" then dv = @layer_name[0,3] + @user_name[0,2] end
        if @ans[0] ==  "Yes" and @ans[1] == "No" then dv = @layer_name[0,3] + @user_name end
        if @ans[0] ==  "No" and @ans[1] == "Yes" then dv = @user_name[0,6] end
        if @ans[0] ==  "No" and @ans[1] == "No" then dv = @user_name end
        ts = "Component Page Name"
        pn = self.get_user_name(ps,dv,ts)
      
        # Test Page name to determine if unique
        loop = "yes"
        while loop == "yes"
          loop = "no"
          @pages.each {|page|
            if page.name == pn
            UI.messagebox("Page \"#{pn}\" exists,make unique.")
            ps = "Make Page Name Unique!!"
            dv = pn
            ts = "Page Name for Component"
            pn = self.get_user_name(ps,dv,ts)
            loop = "yes"
            end # if page
          }
        end # while
        
        #Get Title for Scene
        ps = "Enter Scene Title "
        dv = @user_name
        ts = "Get Title for Drawing Scene"
        scene_title = get_user_name(ps,dv,ts)
        pd = scene_title
        
        # Add Page & Description for Drawing Scene
        self.add_page(pn,pd)
        
        #Insert copy @ orign
        insert_tran = Geom::Transformation.new [0,0,0]
        c2=@model.active_entities.add_instance(ent_def,insert_tran)
 
        #Find current entity bounds with the largest dim
        selection_bb = entity.bounds
        self.get_bounds_max_dir(selection_bb)
        self.get_trans
      
        # Transform and rotate component so largest dim is on the red axis
        if @max_dir != "red axis" then c2.transform! @f_tran  end
        
        
        # Place copy on new layer
        c2.layer=(@layers[@layer_name])
        
        #Move to new location out of view of Model
        offset = @ans[2].to_l
        f_tran = Geom::Transformation.new [offset, 0, offset]
        c2.transform! f_tran
        
        #Add Title as Text to Scene
        
        name_text = @ents.add_text scene_title, [offset, 0, offset]
        name_text.layer=(@layers[@layer_name])
        
        #Add Title to Page Description
        #description = page.description =  scene_title
      
        #Added statements to make Drawing View
        @ss.clear #Clear selection so only one component will be selected
        @ss.add c2  #Add instance in Scene to selection
        Sketchup.active_model.active_view.camera.perspective = false
        view = Sketchup.active_model.active_view
        view.zoom c2
        c2.layer=(@layers[@layer_name]) #Place on new layer
        
        Sketchup.send_action "viewFront:"
        Sketchup.send_action "pageUpdate:"  #Updates page to show current view
        
        Sketchup.active_model.commit_operation
        
        #This section added to make ortho views is file loaded!!!
        if defined?(JF.generate_ortho_views)
          self.drawing_views
        end # if defined?
    
      end #if results==Yes
      
    end # def comp2_layer_scene_command
    
    ############ Use to create LAYER AND SECENE FOR NESTED COMPONENT
    
    def nested_component_command(obj)
      self.get_model_info
      @entity = obj
      ent_def = @entity.definition
      Sketchup.active_model.close_active() # Closes component opened to select Nested Component!!
      
      # Get Layer Name for selection
      obj_parent = []
      obj_parent = obj.parent.instances
      cur_layer_name = obj_parent[0].layer.name
      cur_layer_prefix = "N" + cur_layer_name[1,3] 
      @nc_name=obj.definition.name
      
      #Set variables to get Nested Component Name
      ps = "Name for Nested Component"
      dv = @nc_name
      ts = "Nested Component Name"
      @nc_name = self.get_user_name(ps,dv,ts)
      obj.definition.name=@nc_name
      
      #Set Layer Name for Nested Component 
      if @ans[0] ==  "Yes" then @layer_name = cur_layer_prefix + @nc_name end
      if @ans[0] ==  "No" then @layer_name = @nc_name end
      
      # Test name to determine if unique
      while @layers[@layer_name]
        UI.messagebox("Layer \"#{@layer_name}\" exists,make unique.")
        ps = "Make Layer Name Unique!!"
        dv = @layer_name
        ts = "Nested Component Layer Name"
        @layer_name = self.get_user_name(ps,dv,ts)
      end
      
      Sketchup.active_model.start_operation "Nested_Comp_Scene" 
      
      self.create_layer
      
      #Insert copy @ orign
      insert_tran = Geom::Transformation.new [0,0,0]
      c2=@model.entities.add_instance(ent_def,insert_tran)
      #model.entities.add_instance(definition, transformation)

      # Find current entity bounds with the largest dim
      selection_bb = @entity.bounds
      self.get_bounds_max_dir(selection_bb)
      
      # Transform and rotate component so largest dim is on the red axis
      if @max_dir != "red axis" then c2.transformation = self.get_trans end
      
      # Place copy on new layer
      c2.layer=(@layers[@layer_name])
      
      # Add page for drawing & Make Nested Component visible on Drawing Page
      # Set variables to get Page name
      cur_pns = []
      cur_pns = @pages.each {|name|}
      ps = "Enter Page Name for Nested Component Scene"
      if @ans[0] ==  "Yes" and @ans[1] == "Yes" then dv = cur_layer_prefix[0,3] + @nc_name[0,2] end
      if @ans[0] ==  "Yes" and @ans[1] == "No" then dv = cur_layer_prefix[0,3] + @nc_name end
      if @ans[0] ==  "No" and @ans[1] == "Yes" then dv = @nc_name[0,6] end
      if @ans[0] ==  "No" and @ans[1] == "No" then dv = @nc_name end
      #dv = cur_layer_prefix[0,3] + @nc_name[0,2]
      ts = "Nested Component Page Name"
      pn = self.get_user_name(ps,dv,ts)
      
      # Test Page name to determine if unique
      loop = "yes"
      while loop == "yes"
        loop = "no"
        @pages.each {|page|
          if page.name == pn
          UI.messagebox("Page \"#{pn}\" exists,make unique.")
          ps = "Make Page Name Unique!!"
          dv = pn
          ts = "Page Name for Nested Component"
          pn = self.get_user_name(ps,dv,ts)
          loop = "yes"
          end # if page
        }
      end #while
      
      #Get Title for Scene
      ps = "Enter Scene Title "
      dv = @nc_name
      ts = "Get Title for Drawing Scene"
      scene_title = get_user_name(ps,dv,ts)
      
      # Use Scene Title as Description for Page
      pd = scene_title
      self.add_page(pn,pd)
      
      #Add Title as Text to Scene and make Drawing View
      name_text = @entities.add_text scene_title, [0.5.inch, 0, -0.5.inch]
      name_text.layer=(@layers[@layer_name])
      
      @ss.clear #Clear selection so only one component will be selected
      @ss.add c2  #Add instance in Scene to selection
      Sketchup.active_model.active_view.camera.perspective = false
      view = Sketchup.active_model.active_view
      view.zoom c2
      Sketchup.send_action "viewFront:"
      Sketchup.send_action "pageUpdate:"  #Updates page to show current view
      
      Sketchup.active_model.commit_operation
      
      #This section added to make ortho views if file loaded!!!
      if defined?(JF.generate_ortho_views)
        self.drawing_views
      end # if defined?
        
    end # def nested_component_command 
    
    ############ Use to create SCENE for SUB ASSEMBLY GROUP
    
    def sub_assembly_command
      self.get_model_info
      ssa=@ss.to_a
      @ss.clear
      @group=@ents.add_group(ssa)
      @groupcopy = @group.copy 
      #groupcopy.transform!(tt)
      sel = Sketchup.active_model.selection
      @groupcopy.name = "SA-"
      @group.explode
      
      # Set variables to get layer name
      ps = "Enter Layer Name for SubAssy Layer"
      dv = @groupcopy.name
      ts = "Get Name for Sub-Assembly Layer"
      @layer_name = self.get_user_name(ps,dv,ts)
      
      # Test name to determine if unique
      while @layers[@layer_name]
        UI.messagebox("Layer \"#{@layer_name}\" exists,make unique.")
        @layer_name = self.get_user_name(ps,dv,ts)
      end
      
      Sketchup.active_model.start_operation "SubAssy_Scene"
      
      self.create_layer
       
      # Add group and individual components to New Layer
      @groupcopy.layer = @layer
      @groupcopy.entities.to_a.each{|e|e.layer = @layer}
      
      # Find current entity bounds with the largest dim
      selection_bb = @groupcopy.bounds
      self.get_bounds_max_dir(selection_bb)
      
      # Transform and rotate component so largest dim is on the red axis
      if @max_dir != "red axis" then @groupcopy.transformation = self.get_trans end
      
      #Get Title for Scene
      ps = "Enter Scene Title "
      dv = @user_name
      ts = "Get Title for Drawing Scene"
      scene_title = get_user_name(ps,dv,ts)
      
      # Use Scene Title as Description for Page
      pd = scene_title
      
      # Add page for drawing & Make SubAssy Group visible on Drawing Page
      pn = @layer_name
      self.add_page(pn,pd)
      
      #Add Title as Text to Scene
      name_text = @entities.add_text scene_title, [0, 0, 0]
      name_text.layer=(@layers[@layer_name])
      
      #This section added to make ortho views if file loaded!!!
      @ss.clear #Clear selection so only one component will be selected
      @ss.add @groupcopy  #Add instance in Scene to selection
      
      Sketchup.active_model.commit_operation
      
      if defined?(JF.generate_ortho_views)
        self.drawing_views
      end # if defined?
        
    end # def sub_assembly_command
    
    ############ Use to create SCENE ONLY for component ALREADY ON component layer  AND/OR
    ############ Use to create SCENE on NEW SCENE LAYER for MIRROR COMPONENT (component was flipped) 
    
    def comp2_Scene_command
    
      self.get_model_info
      entity = @ss.first
      ent_def = entity.definition
      cName=@ss[0].definition.name
      parent = entity.parent
      cur_layer_name = @ss[0].layer.name
      self.read_data
      
      Sketchup.active_model.start_operation "Comp2_Scene"
      
      # This section gets layer for scene
      result = UI.messagebox "Add Scene on NEW Layer? ", MB_YESNO
      if !result then;Sketchup.send_action "selectSelectionTool:";return;end
			if result==6  # Add to new layer
        ps = "Accept or Edit New Layer Name "
        dv = cName
        ts = "Get Name for New Layer"
        @layer_name = self.get_user_name(ps,dv,ts)
        
        # Test name to determine if unique
        while @layers[@layer_name]
          UI.messagebox("Layer \"#{@layer_name}\" exists,make unique.")
          @layer_name = self.get_user_name(ps,@layer_name,ts)
        end # while
        self.create_layer
      else
        @layer_name = cur_layer_name
        new_layer = "no"
      end # if result
      
      #Insert copy
      ent = @ss[0]
      parent = ent.parent
      cdef = ent.definition
      c2 = parent.entities.add_instance(cdef, ent.transformation)  
      
      # Find current entity bounds with the largest dim
      selection_bb = entity.bounds
      self.get_bounds_max_dir(selection_bb)
      self.get_trans
      
      # Transform and rotate component so largest dim is on the red axis
      if @max_dir != "red axis" then c2.transform! @f_tran  end
      
      # Place copy on layer
      c2.layer=(@layers[@layer_name])
      
      if new_layer == "no"
        #Move to new location out of view of Model
        offset = @ans[2].to_l
        f_tran = Geom::Transformation.new [offset, 0, offset]
        c2.transform! f_tran      
      end # if new_layer
      
      # Add page for Scene & Make Component visible
      # Set variables to get Page name
      
      ps = "Enter Scene Name "
      if new_layer == "no"
        if @ans[0] ==  "Yes" and @ans[1] == "Yes" then dv = @layer_name[0,3] + cName[0,2] end
        if @ans[0] ==  "Yes" and @ans[1] == "No" then dv = @layer_name[0,3] + cName end
        if @ans[0] ==  "No" and @ans[1] == "Yes" then dv = cName[0,6] end
        if @ans[0] ==  "No" and @ans[1] == "No" then dv = cName end
        #dv = @layer_name[0,3] + cName[0,2]
      else
        if @ans[1] == "Yes" then dv = @layer_name[0,6] end
        if @ans[1] == "No" then dv = @layer_name end
        #dv = @layer_name
      end # new_layer
      ts = "Scene Name"
      pn = self.get_user_name(ps,dv,ts)
      
      # Test Page name to determine if unique
      loop = "yes"
      while loop == "yes"
        loop = "no"
        @pages.each {|page|
          if page.name == pn
          UI.messagebox("Page \"#{pn}\" exists,make unique.")
          ps = "Make Page Name Unique!!"
          dv = pn
          ts = "Page Name for Component"
          pn = self.get_user_name(ps,dv,ts)
          loop = "yes"
          end # if page
        }
      end #while
      
      #Get Title for Scene
      ps = "Enter Scene Title "
      dv = @user_name
      ts = "Get Title for Drawing Scene"
      scene_title = get_user_name(ps,dv,ts)
      
      # Use Scene Title as Description for Page
      pd = scene_title
      
      self.add_page(pn,pd)
      
      #Add Title as Text to Scene and make Drawing View
      if new_layer == "no"
        name_text = @entities.add_text scene_title, [offset, 0, offset]
      else
        point1 = Geom::Point3d.new 0,0,0
        if @max_dir != "red axis"
          point1.transform! ent.transformation
          point1.transform! @f_tran
          name_text = @entities.add_text scene_title, point1
          name_text.layer=(@layers[@layer_name])
        else
          point1.transform! ent.transformation
          name_text = @entities.add_text scene_title, point1
          name_text.layer=(@layers[@layer_name])
        end # if @max_dir  
      end # if new_layer
      
      
      @ss.clear #Clear selection so only one component will be selected
      @ss.add c2  #Add instance in Scene to selection
      Sketchup.active_model.active_view.camera.perspective = false
      view = Sketchup.active_model.active_view
      view.zoom c2
      Sketchup.send_action "viewFront:"
      Sketchup.send_action "pageUpdate:"  #Updates page to show current view
      
      Sketchup.active_model.commit_operation
      
      #This section added to make ortho views if file loaded!!!
      if defined?(JF.generate_ortho_views)
        self.drawing_views
      end # if defined?
      
    end # comp2_Scene_command
    
    ############ Used to Review &/Or Change User Preferences  
      
    def get_pref_command
    
      # Test for Existance of Data File
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file))
        @ans = Array.new(4)
        f = File.new(@file, "w+") 
        #Debug UI.messagebox"created file"
        # Default Data
        @ans[0] = "Yes"
        @ans[1] = "Yes"
        @ans[2] = "200"
        @ans[3] = "Assembly"
        f.puts @ans
        f.close
      end # if not file
      # Get Existing Preferences
      self.read_data
      prompts = ["Use Number Prefix ","Use Short Scene Name","Offset Scene Distance ","Main View Scene Name"]
      defaults= [@ans[0].strip,@ans[1].strip,@ans[2].strip,@ans[3].strip]
      choices = ["Yes|No","Yes|No","",""] 
      results = inputbox prompts, defaults, choices, "Input Preferences "
      if !results then;Sketchup.send_action "selectSelectionTool:";return;end
      # New User Preferences
      @ans[0] = results[0]
      @ans[1] = results[1]
      @ans[2] = results[2]
      @ans[3] = results[3]
      # Save New User Preferences
      self.write_data
        
    end # get_pref_command
    
  ### Common def section 
  #  
    def get_model_info
    
      @model = Sketchup.active_model
      @layers = @model.layers
      @ss = @model.selection
      @ents = @model.active_entities
      @entities = @model.entities
      @pages = @model.pages
      @count = @pages.count
      @length = @layers.length
      @definitions = @model.definitions
      
    end # get_model_info
    
    def get_user_name(ps,dv,ts)
      
      prompts = [ps]
      defaults= [dv]                
      results = inputbox prompts, defaults, ts
      if !results then;Sketchup.send_action "selectSelectionTool:";return;end
      user_name = results[0]
      return user_name
      
    end # get_user_name
    
    def get_bounds_max_dir(selection_bb)
    
      @max_dir = "blue axis"
      if selection_bb.height > selection_bb.depth
        @max_dir = "green axis"
        if selection_bb.width > selection_bb.height then @max_dir = "red axis"  end
       else
        if selection_bb.width > selection_bb.depth then @max_dir = "red axis" end
      end
      
    end # get_bounds_max_dir
    
    def get_trans
    
      case @max_dir

        when "blue axis"
          @f_tran = Geom::Transformation.rotation [0,0,0], [0,1,0], 90.degrees  
          
        when "green axis"
          @f_tran = Geom::Transformation.rotation [0,0,0], [0,0,1], 90.degrees			
          
      end # case
    
    end # get_trans
    
      
    def create_layer  
      #Debug UI.messagebox"Layer Name  = " + @layer_name
      @layer = @layers.add @layer_name
      pb = @layer.page_behavior = LAYER_IS_HIDDEN_ON_NEW_PAGES
      @model.pages.each { |p| p.set_visibility( @layer, false ) }
    
    end # create_layer
    
    def add_page(pn,pd)
    
      newpage = pn
      @pages.add newpage
      page = @pages [newpage]
      page.description = pd
      page.set_visibility(@layers[@layer_name],true)
      
    end # add_page
    
    def drawing_views
    
        JF.generate_ortho_views
        #Added to place Ortho views on SubAssy layer not active layer    
        number = @ss.count
        i=0
        loop do
          break if i==number
          entity = @model.selection[i]
          entity.layer=(@layers[@layer_name]) 
          i += 1
        end
        Sketchup.send_action "pageUpdate:"  #Updates page to show current view
      
    end # drawing_views
    
    def read_data
    
      #Test for Data File
      @file =  __FILE__.gsub(/.rb$/,'.txt')
      if !(FileTest.exist?(@file))
        get_pref_command
      else
        # Read an array of Input Data from the file 
        # open input data file in read mode	       
        f = File.open(@file, "r")    
        @ans = f.readlines
        # Close the file
        f.close
        for i in 0..2
          @ans[i] = @ans[i].strip
        end # for loop
      end # if !(FileTest
      
    end # read_data
	
    def write_data
    
      # Write an array of Input Data to the file
      # open input data file in write mode	       
      f = File.open(@file, "w")
      f.puts @ans
      # Close the file
      f.close
      
    end # write_data
   
  end # proxy class

  ### RUN ONCE
  #
  unless file_loaded?(File.basename(__FILE__))
  
    ### CONTEXT MENU HANDLER
    #
    UI.add_context_menu_handler do |popup|
      sel = Sketchup.active_model.selection
      unless sel.empty?
        if sel.single_object?
          obj = sel[0]
          if obj.is_a?(Sketchup::ComponentInstance) &&
          obj.parent.is_a?(Sketchup::ComponentDefinition)
            popup.add_item(NC2SCENE) {
              #
              nested_component_command(obj)
              #
            }
          else
            if obj.is_a?(Sketchup::ComponentInstance)
              popup.add_item(C2LAYSCENE) {
                #
                comp2_layer_scene_command
                #
              }
              popup.add_item(C2SCNE) {
                #
                comp2_Scene_command
                #
              }
            end # Single component test
          end # Nested component test
        end # single_object?
        if sel.length >= 2
          popup.add_item(GP2SCENE) {
              #
              sub_assembly_command
              #
            }
        end # sel.length
      end # unless empty?
      
    end # add_context_menu_handler
    
    UI.menu("Plugins").add_item(SETPREF){get_pref_command}
  
    file_loaded(File.basename(__FILE__))
    
  end # RUN ONCE
  
end # module
